/* Name: main.c
 * Project: hid-data, example how to use HID for data transfer
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-11
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c 692 2008-11-07 15:07:40Z cs $
 */

/*
This example should run on most AVRs with only little changes. No special
hardware resources except INT0 are used. You may have to change usbconfig.h for
different I/O pins for USB. Please note that USB D+ must be the INT0 pin, or
at least be connected to INT0 as well.
*/



// Salt added {
//
// USB-IO-Mini firmware
// Customized by Salt <http://homepage1.nifty.com/salt/>
//
// FUSE_H = 0xdd, FUSE_L = 0xe1 : PB5 ͎gȂǁAISP ŏĂB
// FUSE_H = 0x5d, FUSE_L = 0xe1 : PB5 ͎g邯ǁAISP ł͏ĂȂBHVS ȂłB
//
// } Salt added


#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/eeprom.h>

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include "oddebug.h"        /* This is also an example for using debug macros */

// Salt added {
#include "../../../libs-device/osccal.h"
#include "USB-IO-Mini.h"
#define PAYLOAD_SIZE (0x8)
#define USB_IO_MINI_FIRMWARE_VERSION (0x01)
// } Salt added

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[22] = {    /* USB report descriptor */
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, PAYLOAD_SIZE,				// Salt changed from 0x95, 0x80,                    //   REPORT_COUNT (128)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};
/* Since we define only one feature report, we don't use report-IDs (which
 * would be the first byte of the report). The entire report consists of 128
 * opaque data bytes.
 */

/* The following variables store the status of the current data transfer */
static uchar    currentAddress;
static uchar    bytesRemaining;

// Salt added {
static uchar g_ucData[PAYLOAD_SIZE];
static uchar g_ucRegTableADMUX[] = { 0x00, 0x02, 0x03, };	// Vref=Vcc, ADC0, ADC2, ADC3
static uchar g_ucADMUXIndex = 0;
static uchar g_ucReadMode = UIMC_SET_READ_MODE_D4A0;		// default read mode
static uchar g_ADCH[3], g_ADCL[3];
// } Salt added

/* ------------------------------------------------------------------------- */

/* usbFunctionRead() is called when the host requests a chunk of data from
 * the device. For more information see the documentation in usbdrv/usbdrv.h.
 */
uchar   usbFunctionRead(uchar *data, uchar len)
{
	uchar i;	// Salt added
	uchar ucPB;	// Salt added

    if(len > bytesRemaining)
        len = bytesRemaining;
    // Salt deleted eeprom_read_block(data, (uchar *)0 + currentAddress, len);

	// Salt added {
	if (currentAddress == 0) {

		if (g_ucData[0] >= UIMC_ONETIME_COMMAND_BEGIN) {
			
			switch (g_ucData[0]) {		// Write R}hŕ

			case UIMC_READ_VERSION:
				g_ucData[1] = USB_IO_MINI_FIRMWARE_VERSION;
				break;

			case UIMC_READ_DEVICE_ID:
				g_ucData[1] = eeprom_read_byte(ER_DEVICE_ID);
				break;

			case UIMC_READ_REGISTER:
				for (i = 0; i < 7; ++i) {
					g_ucData[i + 1] = *((uchar *) ((((unsigned int) g_ucData[1] << 8) + g_ucData[2]) + i));
				}
				break;
			}
			
			// ʂԂ
			for (i = 0; i < len; ++i) {
				data[i] = g_ucData[currentAddress + i];
			}

			g_ucData[0] = 0x00;	// R}h
		}
		else {
			// ʏ read
			g_ucData[0] = g_ucReadMode;
			
			switch (g_ucReadMode) {
			case UIMC_SET_READ_MODE_D4A0:
			default:
				ucPB = PINB;
				g_ucData[1] = ((ucPB & 0x38) >> 2) | (ucPB & 0x01);
				break;

			case UIMC_SET_READ_MODE_D3A1:
				ucPB = PINB;
				g_ucData[1] = ((ucPB & 0x18) >> 2) | (ucPB & 0x01);
				g_ucData[2] = g_ADCL[0];
				g_ucData[3] = g_ADCH[0];
				break;

			case UIMC_SET_READ_MODE_D2A2:
				ucPB = PINB;
				g_ucData[1] = ((ucPB & 0x08) >> 2) | (ucPB & 0x01);
				g_ucData[2] = g_ADCL[0];
				g_ucData[3] = g_ADCH[0];
				g_ucData[4] = g_ADCL[1];
				g_ucData[5] = g_ADCH[1];
				break;

			case UIMC_SET_READ_MODE_D1A3:
				ucPB = PINB;
				g_ucData[1] = ((ucPB & 0x00) >> 2) | (ucPB & 0x01);
				g_ucData[2] = g_ADCL[0];
				g_ucData[3] = g_ADCH[0];
				g_ucData[4] = g_ADCL[1];
				g_ucData[5] = g_ADCH[1];
				g_ucData[6] = g_ADCL[2];
				g_ucData[7] = g_ADCH[2];
				break;
			}

			// ʂԂ
			for (i = 0; i < len; ++i) {
				data[i] = g_ucData[currentAddress + i];
			}

		}

	}
	// } Salt added

    currentAddress += len;
    bytesRemaining -= len;
    return len;
}

/* usbFunctionWrite() is called when the host sends a chunk of data to the
 * device. For more information see the documentation in usbdrv/usbdrv.h.
 */
uchar   usbFunctionWrite(uchar *data, uchar len)
{
	uchar i;	// Salt added
    if(bytesRemaining == 0)
        return 1;               /* end of transfer */
    if(len > bytesRemaining)
        len = bytesRemaining;
    // Salt deleted eeprom_write_block(data, (uchar *)0 + currentAddress, len);
	// Salt added {
	for (i = 0; i < len; ++i) {
		g_ucData[currentAddress + i] = data[i];
	}
	// } Salt added

    currentAddress += len;
    bytesRemaining -= len;

	// Salt added {
	if (bytesRemaining == 0) {
		
		switch (g_ucData[0]) {

		case UIMC_WRITE_DATA:
			// PORTB = (PORTB & 0x06) | ((g_ucData[1] & 0x0e) << 2) |  (g_ucData[1] & 0x01);
			PORTB = (PORTB & ~(((g_ucData[2] & 0x0e) << 2) | (g_ucData[2] & 0x01))) | (((g_ucData[1] & 0x0e) << 2) | (g_ucData[1] & 0x01));	// Mask ď
			break;


		case UIMC_SET_READ_MODE_D4A0:
			g_ucReadMode = g_ucData[0];
			ADCSRA = 0x07;				// ADC Disable, Prescaler = 1/128
			break;

		case UIMC_SET_READ_MODE_D3A1:
		case UIMC_SET_READ_MODE_D2A2:
		case UIMC_SET_READ_MODE_D1A3:
			g_ucReadMode = g_ucData[0];
			ADCSRA = 0x87;				// ADC Enable, Prescaler = 1/128
			
			g_ucADMUXIndex = 0;								// ADC0 I
			ADMUX = g_ucRegTableADMUX[g_ucADMUXIndex];		// ADC0 I
			ADCSRA |= (1 << ADSC);							// start 
			break;


		case UIMC_WRITE_DEVICE_ID:
			eeprom_write_byte(ER_DEVICE_ID, g_ucData[1]);
			break;

		case UIMC_SET_DIRECTION:
			DDRB = (DDRB & 0x06)   | (((g_ucData[1] & 0x0e) << 2) | (g_ucData[1] & 0x01));	// 0x06 = PB1, PB2 ͏ȂB@USB  D+/D- ɎgĂ̂ŁB
			
			// bit  0 ̂Ƃ͓͂Ȃ̂ŁAPORTB ƂH
			// AvŐݒ肵ǂȂ̂ŁA폜B PORTB = PORTB          & (((g_ucData[1] & 0x0e) << 2) |  (g_ucData[1] & 0x01));

			break;

		}
	}
	// } Salt added

    return bytesRemaining == 0; /* return 1 if this was the last chunk */
}

/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* HID class request */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* since we have only one report type, we can ignore the report-ID */
            // Salt changed from bytesRemaining = 128;
            bytesRemaining = PAYLOAD_SIZE;
            currentAddress = 0;
            return USB_NO_MSG;  /* use usbFunctionRead() to obtain data */
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            /* since we have only one report type, we can ignore the report-ID */
            // Salt changed from bytesRemaining = 128;
            bytesRemaining = PAYLOAD_SIZE;
            currentAddress = 0;
            return USB_NO_MSG;  /* use usbFunctionWrite() to receive data from host */
        }
    }else{
        /* ignore vendor type requests, we don't use any */
    }
    return 0;
}

/* ------------------------------------------------------------------------- */

int main(void)
{
uchar   i;

//calibrateOscillator();

    wdt_enable(WDTO_1S);
    /* Even if you don't use the watchdog, turn it off here. On newer devices,
     * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
     */
    // Salt deleted DBG1(0x00, 0, 0);       /* debug output: main starts */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */
    // Salt deleted odDebugInit();
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();
    sei();
    // Salt deleted DBG1(0x01, 0, 0);       /* debug output: main loop starts */
    for(;;){                /* main event loop */
        // Salt deleted DBG1(0x02, 0, 0);   /* debug output: main loop iterates */
        wdt_reset();
        usbPoll();
        
		// Salt added {
		if (g_ucReadMode >= UIMC_SET_READ_MODE_D3A1) {
			if (!(ADCSRA & (1 << ADSC))) {		// ϊI
			
				// ۑāA
				g_ADCL[g_ucADMUXIndex] = ADCL;
				g_ADCH[g_ucADMUXIndex] = ADCH;
				
				//  pin ɕύX
				++g_ucADMUXIndex;
				if (g_ucADMUXIndex > g_ucReadMode - UIMC_SET_READ_MODE_D3A1) {
					g_ucADMUXIndex = 0;
				}
				ADMUX = g_ucRegTableADMUX[g_ucADMUXIndex];
				
				// start 
				ADCSRA |= (1 << ADSC);
			}
		}
		// } Salt added

    }
    return 0;
}

/* ------------------------------------------------------------------------- */
